from gandai.datastore import Cloudstore
from concurrent.futures import ThreadPoolExecutor 

ds = Cloudstore()

def _copy_search(new_search_key="dev-test-parker"):
    source_search_key = "AXZ-144RV"
    keys = ds.keys(f"searches/{source_search_key}/companies/")
    for key in keys:
        obj = ds[key]
        ds[key.replace(source_search_key, new_search_key)] = obj


def add_companies_to_search(companies, search_key, source='grata') -> None:
    def _add_to_search(company) -> None:
        domain = company['domain']
        key = f"searches/{search_key}/companies/{source}/{domain}"
        ds[key] = company
        
    with ThreadPoolExecutor(max_workers=20) as exec:
        futures = exec.map(_add_to_search, companies)


from dataclasses import asdict, dataclass, field

from dacite import from_dict

from gandai.datastore import Cloudstore

ds = Cloudstore()


@dataclass
class User:
    phone: str
    key: str = field(init=False)

    def __post_init__(self):
        self.key = f"user/{self.phone}"


def user_exists(phone: str) -> bool:
    keys = ds.keys("user")
    return f"user/{phone}" in keys


def find_user(phone: str) -> User:
    if user_exists(phone):
        key = f"user/{phone}"
        return from_dict(User, ds[key])


def create_user(phone: str) -> User:
    user = User(phone=phone)
    ds[user.key] = asdict(user)
    return user



import pandas as pd
import requests


def company_search(phrase: str, page=0, page_size=25):
    cookies = {}

    headers = {}

    json_data = {
        "Filters": {
            "CompanyTypes": [],
            "Industries": [],
            "Locations": [],
            "GrowthRate": {},
            "EmployeeRanges": [],
            "EmployeeCount": {},
            "JobCount": {},
            "GrowthIntent": {},
            "TotalRaised": {},
            "RecentRaised": {},
            "DateOfInvestment": {},
            "LatestRoundRaised": [],
            "LatestValuation": {},
            "LastSimilarWebRank": {},
            "LastSimilarWebRankChangeAbsolute": {},
            "LastSimilarWebRankChangePercentage": {},
            "LastSimilarWebPageViews": {},
            "LastSimilarWebPageViewsChangePercentage": {},
            "FoundingYear": {},
            "SourceCount": {},
            "ExcludeAnyTag": False,
            "ExcludeTags": [],
            "IncludeAnyTag": False,
            "IncludeTags": [],
            "Investors": [],
            "CompaniesLikeThese": [],
            "InvestmentAum": {},
            "InvestmentTypes": [],
            "InvestmentStages": [],
            "CustomFields": [],
            "IncludeOutOfBusiness": False,
            "IncludeIncomplete": False,
        },
        "SearchText": phrase,
        "Offset": page,
        "OrderBy": "",
        "Limit": page_size,
        "NewMatchesOnly": False,
        "NewMatchesPeriodInDays": None,
        "SearchWebtext": False,
    }

    response = requests.post(
        "https://www.sourcescrub.com/api/companies/search",
        cookies=cookies,
        headers=headers,
        json=json_data,
    )
    data = response.json()
    print(f"showing {len(data['Items'])} of {data['TotalItemsCount']} companies")

    return pd.DataFrame(data["Items"])

    # Note: json_data will not be serialized by requests
    # exactly as it was in the original request.
    # data = '{"Filters":{"CompanyTypes":[],"Industries":[],"Locations":[],"GrowthRate":{},"EmployeeRanges":[],"EmployeeCount":{},"JobCount":{},"GrowthIntent":{},"TotalRaised":{},"RecentRaised":{},"DateOfInvestment":{},"LatestRoundRaised":[],"LatestValuation":{},"LastSimilarWebRank":{},"LastSimilarWebRankChangeAbsolute":{},"LastSimilarWebRankChangePercentage":{},"LastSimilarWebPageViews":{},"LastSimilarWebPageViewsChangePercentage":{},"FoundingYear":{},"SourceCount":{},"ExcludeAnyTag":false,"ExcludeTags":[],"IncludeAnyTag":false,"IncludeTags":[],"Investors":[],"CompaniesLikeThese":[],"InvestmentAum":{},"InvestmentTypes":[],"InvestmentStages":[],"CustomFields":[],"IncludeOutOfBusiness":false,"IncludeIncomplete":false},"SearchText":"pet food","Offset":0,"OrderBy":"","Limit":50,"NewMatchesOnly":false,"NewMatchesPeriodInDays":null,"SearchWebtext":false}'
    # response = requests.post('https://www.sourcescrub.com/api/companies/search', cookies=cookies, headers=headers, data=data)


# def feature_query():
#     df = pd.read_feather("data/sourcescrub_10k.feather")[
#         0:1000
#     ]  # normalize to grata count
#     scrub_map = {
#         "InformalName": "name",
#         "Domain": "domain",
#         "Description": "description",
#         "EmployeeCount": "employee_count",
#         "LastFinancialRevenue": "rev_millions",
#     }
#     df = df.rename(columns=scrub_map)
#     df = df[scrub_map.values()]
#     df["rev_millions"] = df["rev_millions"].dropna().apply(lambda x: x / (10**6))
#     df["rev_millions"] = df["rev_millions"].fillna(0)
#     df.insert(0, "_source", "scrub")
#     df = df[df["employee_count"] < 1000]
#     df = df[df["employee_count"] > 10]
#     return df



from dataclasses import asdict, dataclass, field
from enum import Enum
from time import time
from typing import Optional

from gandai.datastore import Cloudstore

ds = Cloudstore()


class EventType(str, Enum):
    CREATE = "create"
    ADVANCE = "advance"
    QUALIFY = "qualify"
    REJECT = "reject"
    # FoobarEnum


@dataclass
class Event:
    key: str = field(init=False)
    search_key: str
    # target_key: Optional[str]  # null for a query event
    domain: str

    actor_key: str  #
    ## todo: enums in dataclasses
    type: str  # build, advance, qualify, reject, conflict
    created: int = field(init=False)

    def __post_init__(self):
        # build, advance, qualify, reject, conflict
        self.created = int(time())
        self.key = f"searches/{self.search_key}/events/{self.created}"


def post_event(actor_key: str, search_key: str, domain: str, type: str) -> None:
    event = Event(actor_key=actor_key, search_key=search_key, domain=domain, type=type)
    ds[event.key] = asdict(event)
    return f"posted: {event.key}"



from gandai.datastore import Cloudstore
import requests

ds = Cloudstore()

GRATA_TOKEN = ds['env/GRATA_TOKEN']

def _search_similiar(similar_companies: list, page: int = 1, page_size: int = 25):

    headers = {"authorization": GRATA_TOKEN}

    json_data = {
        "filters": {
            "similar_companies": {
                "op": "and",
                "conditions": [
                    {
                        "include": similar_companies,
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "tolerance": 100,
        },
        "page": page,
        "page_size": page_size,
        "paging": True,
        "query": "",
    }

    response = requests.post(
        "https://search.grata.com/api/search/", headers=headers, json=json_data
    )
    data = response.json()
    return data


def company_search(phrase: str, page=1, page_size=25):

    headers = {"authorization": GRATA_TOKEN}

    json_data = {
        "filters": {
            "keywords": {
                "op": "and",
                "conditions": [
                    {
                        "include": [
                            phrase,
                        ],
                        "exclude": [],
                        "op": "any",
                        "match": "core",
                        "weight": 3,
                        "type": "filter",
                    },
                ],
            },
            "business_models": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "target_verticals": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "sectors": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "lists": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "company_names": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "matching_company_names": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "similar_companies": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "revenue": [],
            "web_traffic": [],
            "employees_range": [],
            "estimated_employees_range": [],
            "employees_change": [],
            "employees_change_time": "annual",
            "web_traffic_change": [],
            "web_traffic_change_time": "annual",
            "employee_growth_anomaly": False,
            "locations": {
                "op": "and",
                "conditions": [
                    {
                        "include": [
                            {
                                "location": {
                                    "lat": 30.98239,
                                    "lon": -98.70754,
                                },
                                "city_name": None,
                                "population": 319929,
                                "region_iso": None,
                                "region_name": None,
                                "country_iso2": "US",
                                "country_iso3": "USA",
                                "country_name": "UNITED STATES",
                                "macro_region": "AMERICAS",
                                "micro_region": "NORTHERN AMERICA",
                                "census_region": None,
                                "continent_name": "NORTH AMERICA",
                                "geography_type": 3,
                                "census_subregion": None,
                            },
                            {
                                "continent_name": "North America",
                                "country_name": "Canada",
                                "country_iso3": "CAN",
                                "location": {
                                    "lat": 58.341159,
                                    "lon": -112.530919,
                                },
                                "sort_rank": 2,
                            },
                        ],
                        "exclude": [],
                        "op": "any",
                        "types": [
                            "hq",
                        ],
                        "order": [
                            "USA",
                            "Canada",
                        ],
                    },
                ],
            },
            "locations_radius": 0,
            "locations_count": [],
            "page_section": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "tolerance": 100,
            "allow_foreign_language": False,
            "tags": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "last_ownership_event": [],
            "latest_funding_date": [],
            "total_funding_amount": [],
            "latest_funding_amount": [],
            "hiring_titles": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "hiring_recency": [],
            "hiring_locations": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "hiring_keywords": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "hiring": False,
            "expansion_terms": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "user_viewed_companies": "off",
            "user_exported_companies": "off",
            "user_synced_companies": "off",
            "user_added_companies": "off",
            "user_annotated_companies": "off",
            "company_age": [],
            "ultimate_owners": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "investors": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "derived_ownership": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                    },
                ],
            },
            "funding_rounds_count": [],
            "naics": {
                "op": "and",
                "conditions": [
                    {
                        "include": [],
                        "exclude": [],
                        "op": "any",
                        "ignore_include": [],
                        "ignore_exclude": [],
                    },
                ],
            },
        },
        "page": page,
        "page_size": page_size,
        "query": "",
    }

    response = requests.post(
        "https://search.grata.com/api/search/",
        headers=headers,
        json=json_data,
    )
    data = response.json()
    return data
    # print(f"showing {len(data['companies'])} of {data['count']} companies")
    # return pd.DataFrame(data['companies'])


# def feature_query() -> pd.DataFrame:
#     def get_rev_m(x):
#         rev_map = {
#             "< 1M": 1,
#             "< 5M": 3,
#             "5 - 10M": 7,
#             "10 - 25M": 17,
#             "25 - 50M": 33,
#             "50 - 100M": 75,
#             "100 - 150M": 125,
#             "150 - 250M": 200,
#             "250 - 500M": 375,
#             "500 - 1B": 750,
#             "1B+": 1500,
#         }
#         if x in rev_map.keys():
#             return rev_map[x]
#         elif pd.isna(x) == False:
#             rev_millions = int(x) / (10**6)
#             # normalized cap to "1B+"
#             # return min(rev_millions, 1500)
#             return rev_millions
#         else:
#             return None

#     df = pd.read_csv("data/grata_1k.csv")
#     df["employee_count"] = df["employees"].apply(
#         lambda x: eval(x).get("value")
#     )  # from the save as csv thing
#     df["rev_millions"] = df["revenue"].apply(get_rev_m)
#     df["lat"] = df["headquarters"].dropna().apply(lambda x: x["location"]["lat"])
#     df["lon"] = df["headquarters"].dropna().apply(lambda x: x["location"]["lon"])

#     grata_cols = [
#         "name",
#         "domain",
#         "description",
#         # "rev_millions",
#         "employee_count",
#         "lat",
#         "lon",
#     ]
#     df = df[grata_cols]
#     df.insert(0, "_source", "grata")
#     df = df[df["employee_count"] < 1000]
#     df = df[df["employee_count"] > 10]
#     df = df[df["rev_millions"] < 1000]  # under 1B
#     return df


# def _get_features(companies: pd.DataFrame) -> pd.DataFrame:
#     df = companies
#     df["employee_count"] = df["employees"].apply(
#         lambda x: x.get("value")
#     )  # from the save as csv thing
#     # df["rev_millions"] = df["revenue"].apply(get_rev_m)
#     df["lat"] = df["headquarters"].dropna().apply(lambda x: x["location"]["lat"])
#     df["lon"] = df["headquarters"].dropna().apply(lambda x: x["location"]["lon"])

#     grata_cols = [
#         "name",
#         "domain",
#         "description",
#         # "rev_millions",
#         "primary_business_model_name",
#         "employee_count",
#         "lat",
#         "lon",
#     ]
#     df = df[grata_cols]
#     df.insert(0, "_source", "grata")
#     # df = df[df["employee_count"] < 1000]
#     # df = df[df["employee_count"] > 10]
#     # df = df[df["rev_millions"] < 1000]  # under 1B
#     return df



from dataclasses import asdict, dataclass, field
from time import time

from gandai.datastore import Cloudstore

ds = Cloudstore()


@dataclass
class Comment:
    key: str = field(init=False)
    search_key: str
    actor_key: str  #
    domain: str  # domain
    body: str
    created: int = field(init=False)

    def __post_init__(self):
        self.created = int(time())
        self.key = f"searches/{self.search_key}/comments/{self.created}"


def post_comment(actor_key: str, search_key: str, domain: str, body: str) -> dict:
    comment = Comment(
        actor_key=actor_key, search_key=search_key, domain=domain, body=body
    )
    ds[comment.key] = asdict(comment)
    return asdict(comment)



from gandai.datastore import Cloudstore
from gandai.models import Event
from gandai.services import Query

ds = Cloudstore()


def load_search(search_key: str, actor_key: str = "") -> dict:
    # Event.post_event(
    #     actor_key=actor_key, search_key=search_key, domain="", type="search"
    # )  # hmm really?
    companies = Query.companies_query(search_key)
    events = Query.events_query(search_key)
    comments = Query.comments_query(search_key)

    def _get_comments(domain: str) -> list:
        if len(comments) == 0:
            return []
        return comments.query("domain == @domain").to_dict(orient="records")

    def _get_events(domain: str) -> list:
        if len(events) == 0:
            return []
        return events.query("domain == @domain").to_dict(orient="records")
        

    companies["comments"] = companies["domain"].apply(_get_comments)
    companies["events"] = companies["domain"].apply(_get_events)
    companies["last_event_type"] = companies["events"].apply(
        lambda x: x[-1]["type"] if len(x) > 0 else "created"
    )

    def _companies_by_state(last_event_type: str) -> list:
        df = companies.query("last_event_type == @last_event_type")
        return df.fillna("").to_dict(orient="records")

    
    inbox = _companies_by_state("created")
    review = _companies_by_state("advance")
    qualified = _companies_by_state("qualify")
    rejected = _companies_by_state("reject")

    resp = {
        "search_key": search_key,
        "actor_key": actor_key,
        "meta": {},
        "companies": {
            "inbox": inbox,
            "review": review,
            "qualified": qualified,
            "rejected": rejected,
        },
    }
    return resp





from dataclasses import asdict, dataclass, field
from typing import List

from gandai.models.Search import Search


@dataclass(order=True)
class Target:
    key: str = field(init=False)  # primary key
    search_key: str  #
    domain: str  # unique
    owner: list

    @property
    def current_owner(self):
        return self.owner[0]


def load_targets(search_key: str) -> List[Search]:
    return []



import pandas as pd
import random
from dataclasses import asdict, dataclass, field
from hashlib import md5
from time import time


from dacite import from_dict
from twilio.rest import Client

from gandai.datastore import Cloudstore
# from gandai.models.User import user_exists # create_user 

ds = Cloudstore()
twilio_client = Client(ds["env/TWILIO_APP"], ds["env/TWILIO_TOKEN"])


@dataclass
class Auth:
    key: str = field(init=False)
    phone: str
    code: str  # 6 digit
    expires: int = field(init=False)
    token: str = field(init=False)

    def __post_init__(self):
        assert len(str(self.code)) == 6
        assert len(str(self.phone)) == 10
        SEVEN_DAYS = (7 * 86400)
        self.expires = int(time()) + SEVEN_DAYS
        self.key = f"auth/{self.phone}/{self.expires}"
        self.token = md5(self.key.encode()).hexdigest()


def _send_code(auth: Auth) -> None:
    message = twilio_client.messages.create(
        to=auth.phone,
        from_=ds["env/TWILIO_NUMBER"],
        body=f"G&AI Research login code: {auth.code}",
    )
    print(f"Login Sent to {auth.phone}")


def send_code(phone: str) -> None:
    # Event.create(actor_key=phone)
    auth_code = str(random.randint(100000, 999999))
    auth = Auth(phone=phone, code=auth_code)
    ds[auth.key] = asdict(auth)
    _send_code(auth)


def authenticate(code: int) -> Auth:
    keys = ds.keys("auth/")[1::]
    df = pd.DataFrame(ds.load_async(keys))
    df = df[df['expires'] > int(time())]
    df = df[df['code'] == code]
    if len(df) > 0: 
        # could parse to Auth and back to dict
        return df.to_dict(orient='records')[0]
    else:
        return None


def validate(token: str) -> bool:
    pass



from dataclasses import asdict, dataclass, field
from typing import List


@dataclass(order=True)
class Company:
    key: str = field(init=False)  # primary key
    name: str
    domain: str  # unique
    description: str  # unique
    # revenue: int  # in millions
    # tags: List[str]  # idea affiliated-search_keys as tag
    _match_tokens: List[str]

    sort_index: float = field(init=False, repr=False)

    def __post_init__(self):
        # assert(self.revenue < (10**4))
        self.key = self.domain
        self.sort_index = self.revenue


from gandai.datastore import Cloudstore

ds = Cloudstore()


# def load_all_orgs() -> List[Organization]:
#     keys = ds.keys("organizations/")
#     data = await asyncio.gather(*[get(k) for k in keys])
#     orgs = list(futures)
#     orgs = [from_dict(Organization, org) for org in orgs]
#     return orgs



import pandas as pd

from gandai.datastore import Cloudstore

ds = Cloudstore()


def companies_query(search_key: str) -> pd.DataFrame:
    return pd.read_feather(f"gs://ph-gandai-dev/searches/{search_key}/companies/company_features.feather")
    # keys = ds.keys(f"searches/{search_key}/companies")
    # df = pd.DataFrame(ds.load_async(keys))
    # df["employee_count"] = df["employees"].apply(lambda x: x.get("value"))
    # df = df.sort_values("employee_count", ascending=False).reset_index(drop=True)
    # df = df.insert(0, 'search_key', search_key)
    # return df[["search_key", "name", "domain", "description", "employee_count"]]


def events_query(search_key: str) -> pd.DataFrame:
    keys = ds.keys(f"searches/{search_key}/events")
    return pd.DataFrame(ds.load_async(keys))

def comments_query(search_key: str) -> pd.DataFrame:
    keys = ds.keys(f"searches/{search_key}/comments")
    return pd.DataFrame(ds.load_async(keys))
